%==========================================================================
function [S,p] = SALMON_forecast_Ricker(S0,alpha,beta,sigma,L,returns,T,QET,MC);
%==========================================================================

%--------------------------------------------------------------------------
% Inputs: 
%         S0         [1,1] scalar, starting abundance [# of spawners]
%         alpha      [1,1] scalar or [~5000,1] matrix, Beverton-Holt max recruitment rate parameter [recruits/spawner/yr]
%         beta       [1,1] scalar or [~5000,1] matrix, Beverton-Holt habitat capacity parameter [spawners^-1]
%         sigma      [1,1] scalar or [~5000,1] matrix, standard deviation of environmental variability 
%         L          [1,1] scalar, number of age classes
%         returns    [1,L] matrix, proportion of adults that return to spawn at ages 1-L
%         T          [1,1] scalar, number of years for simulation forecast
%         QET        [1,1] scalar, quasi-extinction threshold [spawners]
%         MC         [1,1] scalar, number of Monte Carlo repetitions (if MC=1 run is deterministic)
%
% Outputs:
%         S          [T,MC] matrix, forecast spawner abundance for T years and MC repetitions
%         p          [1,3or4] matrix, persistence probability for 25, 50, 100 (and 200) years
%--------------------------------------------------------------------------

rand('state',5000);
randn('state',5000);

S = zeros(T,MC);
extant = ones(T,MC);
threshold = QET*S0;

if MC == 1; % Deterministic
       
    S(1:L) = S0; 
    for t = L+1:T;
        spawners = S(t-1:-1:t-L,mc)'*returns';
        S(t) = alpha*spawners*exp(-beta*spawners)*exp(sigma^2/2);
    end;
    p = 1;   

elseif MC > 1 & prod(size(alpha)) > 1; % Parameter uncertainty and environmental variability
    
    draw = fix(nnz(alpha)*rand(MC,1)+1);
        
    for mc = 1:MC;
        S(1:L,mc) = S0;  
        epsilon = sigma(draw(mc))*randn(T,1);
        for t = L+1:T;        
            spawners = S(t-1:-1:t-L,mc)'*returns';
            S(t,mc) = alpha(draw(mc))*spawners.*exp(-beta(draw(mc))*spawners)*exp(epsilon(t));
            S(t,mc) = min( S(t,mc) , 100*S0 );
            if S(t,mc) < threshold; extant(t:T,mc) = 0; break; end;
        end;
    end;
    if T >=  25; p = sum(extant( 25,:)==1)/MC; 
    if T >=  50; p = [p;sum(extant( 50,:)==1)/MC];
    if T >= 100; p = [p;sum(extant(100,:)==1)/MC];
    if T >= 200; p = [p;sum(extant(200,:)==1)/MC];
    end; end; end; end;

elseif MC > 1 & prod(size(alpha)) == 1; % Environmental variability only
  
    epsilon = sigma*randn(T,MC);   
    S(1:L,:) = S0; 
    for t = L+1:T;       
        spawners = (S(t-1:-1:t-L,:)'*returns')';
        S(t,:) = alpha*spawners.*exp(-beta*spawners).*exp(epsilon(t,:));
        extant(t,:) = prod(extant).*S(t,:)>threshold;
        S(t,:) = S(t,:).*extant(t,:);
    end;
        
    if T >=  25; p = sum(extant( 25,:)==1)/MC; 
    if T >=  50; p = [p;sum(extant( 50,:)==1)/MC];
    if T >= 100; p = [p;sum(extant(100,:)==1)/MC];
    if T >= 200; p = [p;sum(extant(200,:)==1)/MC];
    end; end; end; end;

end;